; Loader (single sector)


.386				;386 architecture

vidmem equ 0b800h		;start of VGA text-memory
color equ 7			;color attribute
scrw equ 80*25			;screen words		
eom equ 0			;end of message


_text SEGMENT PUBLIC USE16
    assume cs:_text,ds:_text
    ORG 0H

EntryPoint:

;make a little beep
    mov ax,0E07h
    int 10h

BOOTSEG equ 07C0h			; original address of boot-sector
INITSEG equ 0900h			; we move boot here
ASKSEG	equ 07C0h			; begin of ask1


; the bootstrap loader is loaded at 7c00h by the bios-startup routines, and moves
; itself to address 9000h, and jumps there.
	mov ax,BOOTSEG
	mov ds,ax

;Relocation:

	mov ax,INITSEG
	mov es,ax

	mov cx,256
	sub si,si
	sub di,di
	cld
	rep movsw

	
	lea ax,go
	add ax,16*INITSEG	;Jump to the new address, where the loader is relocated.
	jmp ax			
	

;continue execution from this point
go:
	mov ax,INITSEG
	mov ds,ax
	mov ss,ax
	mov sp,0FFFh

; Print some message

        mov ax,vidmem
	mov es,ax
	call cls
        lea si,msg
	call printstr

; use nop and multiplications in a double loop to achieve delay 
	mov cx,65000d
delay:
   	nop
	mul ax
	push cx
	mov cx,65d
moredelay:
	nop
	mul ax
	loop moredelay
	pop cx
	loop delay

; load the ask1-sectors from the FDD to memory (where the loader was previously located)
; cx is already 0 from the end of the loop

	mov ax,ASKSEG
	mov es,ax		

load_ask1:
	xor ah,ah		; reset Floppy Drive Controller 
	xor dl,dl
	int 13h			

	xor dx,dx		; drive 0, head 0
	mov cl,02h		; sector 2, track 0
	mov bx,0h		; destination address = 0, in ASKSEG
	mov ah,02h		; service 2
	mov al,1		; one sector will be read
	int 13h			; read from floppy
	jnc ok_load_ask1	; ok - continue (no error on reading)
	jmp load_ask1

ok_load_ask1:
	call stop_motor

	
; after everything is loaded, we jump to the code of ask1
	
	mov ax,ASKSEG		; initialize ds for ask1
	mov ds,ax	
	mov ax,16*ASKSEG	; jump to the beginning of ask1
	jmp ax			

;Subroutines

cls proc near		;Clear screen

	mov di,0
	mov al,' '
	mov ah,color
	mov cx,scrw
	rep stosw

	ret

cls endp

printstr proc near	; Print string


	mov di,line

prints_loop:

	cmp byte ptr[si],eom
	je prints_exit
	movsb
	inc di
	jmp prints_loop

prints_exit:

	ret

printstr endp

 ; Turn off the FDD motor.
 
stop_motor:
	push dx
	mov dx,03f2h
	xor al, al
	out dx,al
	pop dx
	ret

; Since we have a .COM file, data is stored together with code.

line dw  160d
msg  db 'Loading...'
     db eom

_text ENDS

END EntryPoint	
		; End assembly
